###########################################################
## Libraries
###########################################################
library(monoProc)
library(XLConnect)
library(xcms)
library(plyr)
###########################################################



###########################################################
## Some functions we use later
###########################################################
eval_mapping = function(grouped){
  peaktable = peakTable(grouped)
  eval_summary=c()
  
  eval_summary$groups=nrow(grouped@groups)
  eval_summary$avg_width=mean(peaktable[,'rtmax']-peaktable[,'rtmin'])
  
  ## NA values are how many features have NA in either set
#   NA_features = !(peaktable[,'Halle']==1  &   peaktable[,'Copenhagen']==1) 
#   eval_summary$nan_abs=  sum(NA_features)
#   eval_summary$nan_rel= (sum(NA_features)*100)/nrow(grouped@groups)
  
  
  ## NA values are how many in cph set that are unmatched
#   eval_summary$nan_abs=  sum(peaktable[,'Halle']==0  &   peaktable[,'Copenhagen']==1)
#   eval_summary$nan_rel= (eval_summary$nan_abs*100)/sum(peaktable[,'Copenhagen']==1)
  
  
  
  ## How many in cph are matched
  eval_summary$matched_abs=  sum((peaktable[,'Halle']==1  &   peaktable[,'Copenhagen']==1) )
  eval_summary$matched_rel= (sum((peaktable[,'Halle']==1  &   peaktable[,'Copenhagen']==1) )*100)/sum(peaktable[,'Copenhagen']==1)
  
  
  
  
  return(eval_summary)
}


eval_table = function(xset,xset_org,bw,span=0.2,ppm_lim=30){

  result_table = matrix(NA, nrow = length(bw), ncol = 8)
  colnames(result_table) = c('bw_set','# groups','# Matched','% Matched','avg. width','Mean RT dev.','Median RT dev.','90 percentile RT dev.')
  
  
  
  for (i in 1:length(bw)){
    grouped = group(xset,mzwid = 0.02,minsamp=1,minfrac=1,    bw=bw[i])
    result = eval_mapping(grouped)
    
    
    matched_rt=rtMatch_ByIndex(xset_org,grouped)
    matched=!is.na(matched_rt$result_rt[,1]) & !is.na(matched_rt$result_rt[,2])
    ppm=(abs(matched_rt$result_mz[,1]-matched_rt$result_mz[,2])/matched_rt$result_mz[,1])*1E6<ppm_lim
    
    
    fit = loess(formula = matched_rt$result_rt[matched & ppm,1]~matched_rt$result_rt[matched & ppm,2], span=span, degree=2, iterations=10, trace.hat = c("exact"))
    fit_mono_pred = monoproc(fit, bandwidth = 0.0001, mono1 = "increasing", gridsize=1000,xx=matched_rt$result_rt[matched & ppm,2])
    pred_y    = fit_mono_pred@fit@y
    

    
    result_table[i,'bw_set'] = bw[i]
    result_table[i,'# groups'] = result$groups
    result_table[i,'# Matched'] = result$matched_abs
    result_table[i,'% Matched'] = result$matched_rel
    result_table[i,'avg. width'] = result$avg_width
result_table[i,'Mean RT dev.'] =              mean(abs(matched_rt$result_rt[matched & ppm,1]-pred_y))
result_table[i,'Median RT dev.'] =          median(abs(matched_rt$result_rt[matched & ppm,1]-pred_y))
result_table[i,'90 percentile RT dev.'] = quantile(abs(matched_rt$result_rt[matched & ppm,1]-pred_y),0.9)
  }
  
  return(result_table)
}




rtMatch_ByIndex = function(xset,xset_grouping){
result=list()
result_rt = matrix(data=NA,nrow=length(xset_grouping@groupidx), ncol=2)
result_mz = matrix(data=NA,nrow=length(xset_grouping@groupidx), ncol=2)

samp_list = xset_grouping@peaks[,"sample"]

for (i in 1:length(xset_grouping@groupidx)){
  groupidx = xset_grouping@groupidx[[i]]
  group_vec = as.logical(matrix(data=FALSE,nrow(xset_grouping@peaks)))
  group_vec[groupidx]=TRUE
  
  result_rt[i,1] = mean(xset@peaks[samp_list==1 & group_vec,"rt"])
  result_rt[i,2]=  mean(xset@peaks[samp_list==2 & group_vec,"rt"])
  
  result_mz[i,1] = mean(xset@peaks[samp_list==1 & group_vec,"mz"])
  result_mz[i,2]=  mean(xset@peaks[samp_list==2 & group_vec,"mz"])
}

result$result_rt=result_rt
result$result_mz=result_mz
return(result)
}
###########################################################





##############################################
## Get the data
##############################################
# Load halle data
load("xset_halle_office_plasma")
xset_halle_office_plasma@phenoData[1] = 'Halle'
rownames(xset_halle_office_plasma@phenoData) = 'Halle'

# Get cph data
load("xset_plasma_pos")
xset_cph_office_plasma@phenoData[1] = 'Copenhagen'
rownames(xset_cph_office_plasma@phenoData) = 'Copenhagen'

##############################################




##############################################
## Correct mass in cph set
##############################################
#median mass error was found to be 11 ppm between cph and Copenhagen. We "correct" this offset to get better matching
xset_cph_office_plasma@peaks[,"mz"]    = xset_cph_office_plasma@peaks[,"mz"]    - (11/1E6)*xset_cph_office_plasma@peaks[,"mz"]
xset_cph_office_plasma@peaks[,"mzmin"] = xset_cph_office_plasma@peaks[,"mzmin"] - (11/1E6)*xset_cph_office_plasma@peaks[,"mzmin"]
xset_cph_office_plasma@peaks[,"mzmin"] = xset_cph_office_plasma@peaks[,"mzmin"] - (11/1E6)*xset_cph_office_plasma@peaks[,"mzmin"]
##############################################




##############################################
## Stats before doing any correction
##############################################
xset_merged_office_plasma_original = c(xset_halle_office_plasma,xset_cph_office_plasma)

bw = c(10,30,60,180,360)
map_sum_direct = eval_table(xset_merged_office_plasma_original,xset_merged_office_plasma_original,bw=bw,span=0.8)
##############################################




##############################################
## Stats after streatching by run time
##############################################

rt_max = c()
for (i in 1:length(xset_merged_office_plasma_original@filepaths)){
xraw = xcmsRaw(xset_merged_office_plasma_original@filepaths[i])
rt_max[i] = max(xraw@scantime)
rm(xraw)
}

rt_factor = rt_max[1]/rt_max[2]



xset_cph_office_plasma_stretch = xset_cph_office_plasma

for (i in 1:nrow(xset_cph_office_plasma@phenoData)) {
  xset_cph_office_plasma_stretch@rt$raw[[i]] = xset_cph_office_plasma@rt$raw[[i]]               *rt_factor
  xset_cph_office_plasma_stretch@rt$corrected[[i]] = xset_cph_office_plasma@rt$corrected[[i]]   *rt_factor
}

xset_cph_office_plasma_stretch@peaks[,"rt"] = xset_cph_office_plasma@peaks[,"rt"]           *rt_factor
xset_cph_office_plasma_stretch@peaks[,"rtmin"] = xset_cph_office_plasma@peaks[,"rtmin"]     *rt_factor
xset_cph_office_plasma_stretch@peaks[,"rtmax"] = xset_cph_office_plasma@peaks[,"rtmax"]     *rt_factor




bw = c(10,30,60,180,360)
map_sum_stretch = eval_table(c(xset_halle_office_plasma,xset_cph_office_plasma_stretch),xset_merged_office_plasma_original,bw,span=0.2)
rm(bw)

rm(rt_max,rt_factor)
##############################################






##############################################
## "correct" data in cph samples
##############################################
#source(1ret_lowess.r, echo=TRUE)
xset_cph_office_plasma_simple_map = xset_cph_office_plasma


for (i in 1:nrow(xset_cph_office_plasma@phenoData)) {
  fit_mono_pred = monoproc(fit, bandwidth = 0.0001, mono1 = "increasing", gridsize=1000,xx=xset_cph_office_plasma@rt$raw[[i]])
  pred_y    = fit_mono_pred@fit@y

  
  xset_cph_office_plasma_simple_map@rt$raw[[i]] = pred_y
  xset_cph_office_plasma_simple_map@rt$corrected[[i]] = pred_y
}


fit_mono_pred = monoproc(fit, bandwidth = 0.0001, mono1 = "increasing", gridsize=1000,xx=xset_cph_office_plasma@peaks[,"rt"])
pred_y    = fit_mono_pred@fit@y
xset_cph_office_plasma_simple_map@peaks[,"rt"] = pred_y

fit_mono_pred = monoproc(fit, bandwidth = 0.0001, mono1 = "increasing", gridsize=1000,xx=xset_cph_office_plasma@peaks[,"rtmin"])
pred_y    = fit_mono_pred@fit@y
xset_cph_office_plasma_simple_map@peaks[,"rtmin"] = pred_y

fit_mono_pred = monoproc(fit, bandwidth = 0.0001, mono1 = "increasing", gridsize=1000,xx=xset_cph_office_plasma@peaks[,"rtmax"])
pred_y    = fit_mono_pred@fit@y
xset_cph_office_plasma_simple_map@peaks[,"rtmax"] = pred_y




bw = c(10,30,60,180,360)
map_sum_simple_map = eval_table(c(xset_halle_office_plasma,xset_cph_office_plasma_simple_map),xset_merged_office_plasma_original,bw,span=0.2)
rm(bw)

##############################################



##############################################
## merge data
##############################################
xset_simple_map_merged = c(xset_halle_office_plasma,xset_cph_office_plasma_simple_map)
##############################################


##############################################
## Retention time correction
##############################################
# remove Halle sample... for debugging
#xset_merged_office_plasma = split(xset_merged_office_plasma,xset_merged_office_plasma@phenoData$class)$data_converted_pos

xset_simple_map_merged_g = group(xset_simple_map_merged, minsamp=1, minfrac = 1, mzwid = 0.02, bw=30)

xset_simple_map_merged_g_r    = retcor(xset_simple_map_merged_g, plottype="deviation",
                                          span = 0.2, missing = 0, extra = 1, col=c(2,rep(1,2)), ty=rep(2,nrow(xset_simple_map_merged_g@phenoData)))


bw = c(10,30,60,180,360)
map_sum_full_map = eval_table(xset_simple_map_merged_g_r,xset_merged_office_plasma_original,bw,span=0.1)
rm(bw)



xset_simple_map_merged_g_r_g = group(xset_simple_map_merged_g_r, minsamp=1, minfrac = 1, mzwid = 0.02, bw=30)

#xset_merged_office_plasma_g_r_g_r    = retcor(xset_merged_office_plasma_g_r_g, plottype="deviation",
#                                          span = 0.2, missing = 0, extra = 1, col=c(2,rep(1,16)), ty=rep(16,nrow(xset_merged_office_plasma_g@phenoData)))




##############################################



##############################################
## Getting matching ret times
##############################################
result=rtMatch_ByIndex(xset_merged_office_plasma_original,xset_simple_map_merged_g_r_g)

ppm = ((result$result_mz[,1]-result$result_mz[,2])/result$result_mz[,1])*1E6
ppm_lim= 30
plot(result$result_rt[abs(ppm)<ppm_lim,2],result$result_rt[abs(ppm)<ppm_lim,1], main="Cph to Halle retention time",xlab="RT (Cph)", ylab="RT (Halle)", pch=20)
points(result$result_rt[abs(ppm)>ppm_lim,2],result$result_rt[abs(ppm)>ppm_lim,1],col=2, pch=20)

plot(result$result_rt[,2],ppm, xlab="mz", ylab="ppm", main="Cph to Halle retention time, ppm", pch=20)

##############################################


##############################################
## Make fit to Office plasma
##############################################
## Make normal and monotonic fit
fit_office = loess(formula = result$result_rt[abs(ppm)<ppm_lim,1]~result$result_rt[abs(ppm)<ppm_lim,2], span=0.1, degree=2, iterations=10, trace.hat = c("exact"))
fit_office_mono = monoproc(fit_office, bandwidth = 0.0001, mono1 = "increasing", gridsize=1000)
###########################################################



###########################################################
## Plot the fits
###########################################################
plot(result$result_rt[abs(ppm)<ppm_lim,2]/60,result$result_rt[abs(ppm)<ppm_lim,1]/60, main="Retention time conversion between systems",xlab="RT (Cph)", ylab="RT (Halle)", pch=20)
lines(fit_office_mono@fit@x/60,fit_office_mono@fit@y/60,col=2)


#plot(fit_office,main="Cph to Halle retention time",xlab="RT (Cph)", ylab="RT (Halle)", pch=20)
#ret_halle_pred = predict(fit_office, seq(from=0, to=max(result_rt[abs(ppm)<ppm_lim,1],na.rm=TRUE), by=0.1))
#lines(seq(from=0, to=max(result_rt[abs(ppm)<ppm_lim,1],na.rm=TRUE), by=0.1),result_rt[abs(ppm)<ppm_lim,2])
#rm(ret_halle_pred)
###########################################################




###########################################################
## plot deviation
###########################################################
fit_mono_pred = monoproc(fit_office, bandwidth = 0.0001, mono1 = "increasing", gridsize=1000,xx=result$result_rt[(abs(ppm)<ppm_lim) & (!is.na(ppm)),2])
pred_y    = fit_mono_pred@fit@y
plot(result$result_rt[(abs(ppm)<ppm_lim) & (!is.na(ppm)),1],result$result_rt[(abs(ppm)<ppm_lim) & (!is.na(ppm)),1]-pred_y, main="Deviation",pch=20)
###########################################################




###########################################################
## save the fit
###########################################################
#save(fit_office, fit_office_mono,file="models/fit_office")
###########################################################